@@ -64,11 +64,12 @@ FDAModel <- function(
     label = "Flexible Discriminant Analysis",
     packages = "mda",
     response_types = "factor",
+    weights = TRUE,
     predictor_encoding = "model.matrix",
-    params = params(environment(), ...),
+    params = new_params(environment(), ...),
     gridinfo = new_gridinfo(
       param = c("nprune", "degree"),
-      values = c(
+      get_values = c(
         function(n, data, ...) {
           model_fit <- fit(data, model = EarthModel(pmethod = "none"))
           max_terms <- min(2 + 0.75 * nrow(model_fit$dirs), 200)
@@ -106,14 +107,14 @@ MLModelFunction(FDAModel) <- NULL
 #' }
 #'
 PDAModel <- function(lambda = 1, df = NULL, ...) {
-  args <- params(environment(), ...)
+  args <- new_params(environment(), ...)
   args$method <- .(mda::gen.ridge)
   model <- do.call(FDAModel, args, quote = TRUE)
   model@name <- "PDAModel"
   model@label <- "Penalized Discriminant Analysis"
   model@gridinfo <- new_gridinfo(
     param = "lambda",
-    values = c(
+    get_values = c(
       function(n, ...) c(0, 10^seq_inner(-5, 1, n - 1))
     )
   )
