@@ -46,7 +46,7 @@ GLMBoostModel <- function(
   stopintern = FALSE, trace = FALSE
 ) {
 
-  args <- params(environment())
+  args <- new_params(environment())
   is_main <- names(args) %in% "family"
   params <- args[is_main]
   params$control <- as.call(c(.(mboost::boost_control), args[!is_main]))
@@ -57,11 +57,12 @@ GLMBoostModel <- function(
     packages = "mboost",
     response_types = c("binary", "BinomialVariate", "NegBinomialVariate",
                        "numeric", "PoissonVariate", "Surv"),
+    weights = TRUE,
     predictor_encoding = "model.frame",
     params = params,
     gridinfo = new_gridinfo(
       param = "mstop",
-      values = c(
+      get_values = c(
         function(n, ...) round(seq_range(0, 50, c(1, 1000), n + 1))
       )
     ),
@@ -84,12 +85,12 @@ GLMBoostModel <- function(
                matrix = mboost::glmboost(x, y, weights = weights,
                                          family = family, ...))
     },
-    predict = function(object, newdata, ...) {
+    predict = function(object, newdata, model, ...) {
       newdata <- as.data.frame(newdata)
       if (object$family@name == "Cox Partial Likelihood") {
         lp <- drop(predict(object, type = "link"))
         new_lp <- drop(predict(object, newdata = newdata, type = "link"))
-        predict(object$response, lp, new_lp, ...)
+        predict(object$response, lp, new_lp, weights = case_weights(model), ...)
       } else {
         predict(object, newdata = newdata, type = "response")
       }
