@@ -3,6 +3,6 @@ NullModel <- function() {
     name = "NullModel",
     label = "Null Model",
     response_types = settings("response_types"),
-    fit = function(...) stop("no specified model to fit", call. = FALSE)
+    fit = function(...) throw(LocalError("no specified model to fit"))
   )
 }
