@@ -102,11 +102,12 @@ RFSRCModel <- function(
     label = "Random Forest (SRC)",
     packages = "randomForestSRC",
     response_types = c("factor", "matrix", "numeric", "Surv"),
+    weights = TRUE,
     predictor_encoding = "model.frame",
-    params = params(environment()),
+    params = new_params(environment()),
     gridinfo = new_gridinfo(
       param = c("mtry", "nodesize"),
-      values = c(
+      get_values = c(
         function(n, data, ...) seq_nvars(data, RFSRCModel, n),
         function(n, data, ...) round(seq(1, min(20, nrow(data)), length = n))
       )
