@@ -31,7 +31,7 @@ SurvRegModel <- function(
 
   dist <- match.arg(dist)
 
-  args <- params(environment(), ...)
+  args <- new_params(environment(), ...)
   is_main <- names(args) %in% c("dist", "scale", "parms")
   params <- args[is_main]
   params$control <- as.call(c(.(survival::survreg.control), args[!is_main]))
@@ -41,6 +41,7 @@ SurvRegModel <- function(
     label = "Parametric Survival",
     packages = c("rms", "Hmisc"),
     response_types = "Surv",
+    weights = TRUE,
     predictor_encoding = "model.matrix",
     params = params,
     fit = function(formula, data, weights, ...) {
@@ -103,7 +104,7 @@ SurvRegStepAICModel <- function(
 
   direction <- match.arg(direction)
 
-  args <- params(environment())
+  args <- new_params(environment())
   is_step <- names(args) %in% c("direction", "scope", "k", "trace", "steps")
   params <- args[is_step]
 
@@ -114,6 +115,7 @@ SurvRegStepAICModel <- function(
     label = "Parametric Survival (Stepwise)",
     packages = c(stepmodel@packages, "MASS"),
     response_types = stepmodel@response_types,
+    weights = stepmodel@weights,
     predictor_encoding = stepmodel@predictor_encoding,
     params = c(stepmodel@params, params),
     fit = function(formula, data, weights, direction = "both", scope = list(),
