@@ -46,8 +46,9 @@ TreeModel <- function(
     label = "Regression and Classification Trees",
     packages = "tree",
     response_types = c("factor", "numeric"),
+    weights = TRUE,
     predictor_encoding = "model.frame",
-    params = params(environment()),
+    params = new_params(environment()),
     fit = function(formula, data, weights, split, k = NULL, best = NULL,
                    method = NULL, ...) {
       model_fit <- tree::tree(formula, data = as.data.frame(data),
