@@ -7,7 +7,10 @@
 #' @rdname ModelFrame-methods
 #'
 #' @param x model \code{\link{formula}} or \code{\link{matrix}} of predictor
-#'   variables.
+#'   variables.  In the case of a formula, arguments \code{weights} and
+#'   \code{strata} are evaluated as expressions, whose objects are searched for
+#'   first in the accompanying \code{data} environment and, if not found there,
+#'   next in the calling environment.
 #' @param y response variable.
 #' @param data \link[=data.frame]{data frame} or an object that can be converted
 #'   to one.
@@ -16,7 +19,8 @@
 #' @param offsets numeric vector, matrix, or data frame of values to be added
 #'   with a fixed coefficient of 1 to linear predictors in compatible regression
 #'   models.
-#' @param weights vector of case weights [default: equal].
+#' @param weights numeric vector of non-negative case weights for the \code{y}
+#'   response variable [default: equal weights].
 #' @param strata vector of values to use in conducting stratified
 #'   \link{resample} estimation of model performance [default: none].
 #' @param ... arguments passed to other methods.
@@ -31,7 +35,7 @@
 #' ## Requires prior installation of suggested package gbm to run
 #'
 #' mf <- ModelFrame(ncases / (ncases + ncontrols) ~ agegp + tobgp + alcgp,
-#'                  data = esoph, weights = with(esoph, ncases + ncontrols))
+#'                  data = esoph, weights = ncases + ncontrols)
 #' gbm_fit <- fit(mf, model = GBMModel)
 #' varimp(gbm_fit)
 #' }
@@ -68,8 +72,12 @@ ModelFrame.formula <- function(
   }
 
   data <- as.data.frame(data)
+
+  weights <- eval(substitute(weights), data, parent.frame())
+  strata <- eval(substitute(strata), data, parent.frame())
+
   model_terms <- terms(x, data = data)
-  data[[deparse(response(model_terms))]] <- response(model_terms, data)
+  data[[deparse1(response(model_terms))]] <- response(model_terms, data)
   data <- data[all.vars(model_formula(model_terms))]
 
   ModelFrame(model_terms, data, na.rm = na.rm, names = rownames(data),
@@ -99,7 +107,7 @@ ModelFrame.matrix <- function(
     offsets <- tail(names(data), length(offsets))
   }
   model_terms <- terms(x, y, offsets = offsets)
-  data[[deparse(response(model_terms))]] <- y
+  data[[deparse1(response(model_terms))]] <- y
   end <- length(data)
   data <- data[c(end, seq_len(end - 1))]
 
@@ -109,10 +117,9 @@ ModelFrame.matrix <- function(
 
 
 ModelFrame.ModelFrame <- function(x, na.rm = TRUE, ...) {
-  extras <- as.data.frame(Filter(length, list(...)), stringsAsFactors = FALSE)
-  if (length(extras)) {
-    names(extras) <- paste0("(", names(extras), ")")
-    x[names(extras)] <- extras
+  extras <- list(...)
+  for (name in names(extras)) {
+    x[[paste0("(", name, ")")]] <- extras[[name]]
   }
   if (na.rm) na.omit(x) else x
 }
@@ -127,18 +134,20 @@ ModelFrame.ModelTerms <- function(x, data, ...) {
 
 ModelFrame.recipe <- function(x, ...) {
   x <- prep(x)
-  data <- juice(x)
-
-  model_terms <- terms(x)
-  data[[deparse(response(model_terms))]] <- response(model_terms, data)
-  data <- data[all.vars(model_formula(model_terms))]
+  data <- bake(x, NULL)
 
   weights_name <- case_weights_name(x)
+  weights <- if (length(weights_name)) data[[weights_name]]
   strata_name <- case_strata_name(x)
+  strata <- if (length(strata_name)) data[[strata_name]]
+
+  model_terms <- terms(x)
+  data[[deparse1(response(model_terms))]] <- response(model_terms, data)
+  data <- data[all.vars(model_formula(model_terms))]
+
   ModelFrame(model_terms, data, na.rm = FALSE,
              names = if (is.null(data[["(names)"]])) rownames(data),
-             weights = if (length(weights_name)) data[[weights_name]],
-             strata = if (length(strata_name)) data[[strata_name]])
+             weights = weights, strata = strata)
 }
 
 
@@ -166,7 +175,7 @@ inline_calls <- function(x) {
 model_formula <- function(x) {
   x <- formula(x)
   response <- response(x)
-  if (!is.null(response)) x[[2]] <- as.name(deparse(response))
+  if (!is.null(response)) x[[2]] <- as.name(deparse1(response))
   x
 }
 
@@ -220,7 +229,7 @@ terms.list <- function(
     "ModelFormulaTerms"
   }
 
-  var_names <- c(if (has_y) deparse(y), x_char)
+  var_names <- c(if (has_y) deparse1(y), x_char)
   label_names <- x_char[name_inds]
 
   if (class == "ModelDesignTerms") {
@@ -318,8 +327,8 @@ terms.recipe_info <- function(x, ...) {
   if (length(matrix) == 1) matrix <- character()
   other <- setdiff(get_vars("outcome"), c(binom, surv, matrix))
 
-  have_outcome <- as.logical(lengths(list(binom, surv, matrix, other)))
-  if (sum(have_outcome) > 1 || length(other) > 1) {
+  num_roles <- sum(lengths(list(binom, surv, matrix, other)) > 0)
+  if (num_roles > 1 || length(other) > 1) {
     throw(Error("specified outcome is not a single variable, binomial ",
                 "variable with roles 'binom_x' and 'binom_size', survival ",
                 "variables with roles 'surv_time' and 'surv_event', or ",
