@@ -43,7 +43,14 @@ ModelRecipe.recipe <- function(object, ...) {
 
 
 bake.ModelRecipe <- function(object, new_data, ...) {
-  bake(as(object, "recipe"), new_data = prep_recipe_data(new_data))
+  new_data <- if (is.null(new_data)) {
+    object$template
+  } else if (is(new_data, "ModelRecipe")) {
+    new_data$template
+  } else {
+    prep_recipe_data(new_data)
+  }
+  bake(as(object, "recipe"), new_data)
 }
 
 
@@ -57,21 +64,6 @@ bake.TunedInput <- function(object, ...) {
 }
 
 
-juice <- function(x, ...) {
-  UseMethod("juice")
-}
-
-
-juice.default <- function(x, ...) {
-  recipes::juice(x, ...)
-}
-
-
-juice.ModelRecipe <- function(x, ...) {
-  bake(x, x$template)
-}
-
-
 prep.ModelFrame <- function(x, ...) x
 
 
