@@ -8,6 +8,9 @@
 #' @param x \link[=response]{observed responses} or \link{resample} result
 #'   containing observed and predicted responses.
 #' @param y \link[=predict]{predicted responses} if not contained in \code{x}.
+#' @param weights numeric vector of non-negative
+#'   \link[=case_weights]{case weights} for the observed \code{x} responses
+#'   [default: equal weights].
 #' @param breaks value defining the response variable bins within which to
 #'   calculate observed mean values.  May be specified as a number of bins, a
 #'   vector of breakpoints, or \code{NULL} to fit smooth curves with splines for
@@ -35,21 +38,27 @@
 #'
 #' library(survival)
 #'
+#' control <- CVControl() %>% set_predict(times = c(90, 180, 360))
 #' res <- resample(Surv(time, status) ~ ., data = veteran, model = GBMModel,
-#'                 control = CVControl(times = c(90, 180, 360)))
+#'                 control = control)
 #' cal <- calibration(res)
 #' plot(cal)
 #' }
 #'
 calibration <- function(
-  x, y = NULL, breaks = 10, span = 0.75, distr = NULL, na.rm = TRUE, ...
+  x, y = NULL, weights = NULL, breaks = 10, span = 0.75, distr = NULL,
+  na.rm = TRUE, ...
 ) {
   if (na.rm) {
-    complete <- complete_subset(x = x, y = y)
+    complete <- complete_subset(x = x, y = y, weights = weights)
     x <- complete$x
     y <- complete$y
+    weights <- complete$weights
   }
-  .calibration(x, y, breaks = breaks, span = span, distr = distr)
+  Calibration(
+    .calibration(x, y, weights, breaks = breaks, span = span, distr = distr),
+    smoothed = is.null(breaks)
+  )
 }
 
 
@@ -66,59 +75,43 @@ Calibration <- function(object, ..., .check = TRUE) {
 }
 
 
-.calibration <- function(x, ...) {
-  UseMethod(".calibration")
-}
+setGeneric(".calibration",
+  function(observed, predicted, ...) standardGeneric(".calibration")
+)
 
 
-.calibration.default <- function(x, y, breaks, ...) {
-  Calibration(.calibration_default(x, y, breaks = breaks, ...),
-              smoothed = is.null(breaks))
-}
-
-
-.calibration.Resamples <- function(x, ...) {
-  cal_list <- by(x, x$Model, function(data) {
-    calibration(data$Observed, data$Predicted, na.rm = FALSE, ...)
-  }, simplify = FALSE)
-  do.call(c, cal_list)
-}
-
-
-setGeneric(".calibration_default", function(observed, predicted, ...)
-  standardGeneric(".calibration_default"))
-
-
-setMethod(".calibration_default", c("ANY", "ANY"),
+setMethod(".calibration", c("ANY", "ANY"),
   function(observed, predicted, ...) {
     throw(Error("calibration unavailable for response type"))
   }
 )
 
 
-setMethod(".calibration_default", c("factor", "matrix"),
+setMethod(".calibration", c("factor", "matrix"),
   function(observed, predicted, ...) {
-    cal <- calibration(model.matrix(~ observed - 1), predicted, ...)
+    cal <- .calibration(model.matrix(~ observed - 1), predicted, ...)
     bounds <- c("Lower", "Upper")
-    cal$Observed[, bounds] <- pmin(pmax(cal$Observed[, bounds], 0), 1)
+    cal$Observed[, bounds] <- pmin(pmax(0, cal$Observed[, bounds]), 1)
     cal
   }
 )
 
 
-setMethod(".calibration_default", c("factor", "numeric"),
+setMethod(".calibration", c("factor", "numeric"),
   function(observed, predicted, ...) {
     observed <- as.numeric(observed == levels(observed)[2])
-    cal <- calibration(observed, predicted, ...)
+    cal <- .calibration(observed, predicted, ...)
     bounds <- c("Lower", "Upper")
-    cal$Observed[, bounds] <- pmin(pmax(cal$Observed[, bounds], 0), 1)
+    cal$Observed[, bounds] <- pmin(pmax(0, cal$Observed[, bounds]), 1)
     cal
   }
 )
 
 
-setMethod(".calibration_default", c("matrix", "matrix"),
-  function(observed, predicted, breaks, span, ...) {
+setMethod(".calibration", c("matrix", "matrix"),
+  function(observed, predicted, weights, breaks, span, ...) {
+    weights <- check_weights(weights, observed[, 1])
+    throw(check_assignment(weights))
     df <- data.frame(Response = rep(factor(colnames(predicted)),
                                     each = nrow(predicted)),
                      Predicted = as.numeric(predicted))
@@ -126,7 +119,7 @@ setMethod(".calibration_default", c("matrix", "matrix"),
       loessfit_list <- map(function(i) {
         y <- observed[, i]
         x <- predicted[, i]
-        predict(loess(y ~ x, span = span), se = TRUE)
+        predict(loess(y ~ x, weights = weights, span = span), se = TRUE)
       }, seq_len(ncol(predicted)))
       Mean <- c(map_num(getElement, loessfit_list, "fit"))
       SE <- c(map_num(getElement, loessfit_list, "se.fit"))
@@ -137,31 +130,51 @@ setMethod(".calibration_default", c("matrix", "matrix"),
     } else {
       df$Predicted <- midpoints(df$Predicted, breaks)
       df$Observed <- c(observed)
-      aggregate(. ~ Response + Predicted, df, function(x) {
-        Mean <- mean(x)
-        SE <- sd(x) / sqrt(length(x))
-        c(Mean = Mean, SE = SE, Lower = Mean - SE, Upper = Mean + SE)
-      })
+      df$Weight <- weights
+      by_result <- by(df, df[c("Predicted", "Response")], function(data) {
+        Mean <- weighted_mean(data$Observed, data$Weight)
+        SE <- weighted_sd(data$Observed, data$Weight) / sqrt(nrow(data))
+        result <- data[1, c("Response", "Predicted")]
+        result$Observed <- cbind(Mean = Mean, SE = SE,
+                                 Lower = Mean - SE,
+                                 Upper = Mean + SE)
+        result
+      }, simplify = FALSE)
+      do.call(rbind, by_result)
     }
   }
 )
 
 
-setMethod(".calibration_default", c("numeric", "numeric"),
+setMethod(".calibration", c("numeric", "numeric"),
   function(observed, predicted, ...) {
-    calibration(cbind(y = observed), cbind(y = predicted), ...)
+    .calibration(cbind(y = observed), cbind(y = predicted), ...)
   }
 )
 
 
-setMethod(".calibration_default", c("Surv", "SurvProbs"),
-  function(observed, predicted, breaks, ...) {
+setMethod(".calibration", c("Resamples", "ANY"),
+  function(observed, predicted, weights, ...) {
+    cal_list <- by(observed, observed$Model, function(resample) {
+      calibration(resample$Observed, resample$Predicted, resample$Weight,
+                  na.rm = FALSE, ...)
+    }, simplify = FALSE)
+    do.call(c, cal_list)
+  }
+)
+
+
+setMethod(".calibration", c("Surv", "SurvProbs"),
+  function(observed, predicted, weights, breaks, ...) {
+    weights <- check_weights(weights, observed)
+    throw(check_assignment(weights))
     times <- predicted@times
     df <- data.frame(Response = rep(factor(colnames(predicted)),
                                     each = nrow(predicted)),
                      Predicted = as.numeric(predicted))
     if (is.null(breaks)) {
       throw(check_censoring(observed, "right"))
+      throw(check_equal_weights(weights))
       Mean <- c(map_num(function(i) {
         x <- predicted[, i]
         harefit <- polspline::hare(observed[, "time"], observed[, "status"], x)
@@ -172,15 +185,16 @@ setMethod(".calibration_default", c("Surv", "SurvProbs"),
     } else {
       df$Predicted <- midpoints(df$Predicted, breaks)
       df$Observed <- rep(observed, times = length(times))
+      df$Weight <- weights
       df$Time <- rep(times, each = nrow(predicted))
       by_results <- by(df, df[c("Predicted", "Response")], function(data) {
-        km <- survfit(Observed ~ 1, data = data)
+        km <- survfit(Observed ~ 1, data = data, weights = data$Weight)
         interval <- findInterval(data$Time[1], c(0, km$time))
         Mean <- c(1, km$surv)[interval]
         SE <- c(0, km$std.err)[interval]
         result <- data[1, c("Response", "Predicted")]
         result$Observed <- cbind(Mean = Mean, SE = SE,
-                                 Lower = max(Mean - SE, 0),
+                                 Lower = max(0, Mean - SE),
                                  Upper = min(Mean + SE, 1))
         result
       }, simplify = FALSE)
@@ -190,8 +204,10 @@ setMethod(".calibration_default", c("Surv", "SurvProbs"),
 )
 
 
-setMethod(".calibration_default", c("Surv", "numeric"),
-  function(observed, predicted, breaks, distr, span, ...) {
+setMethod(".calibration", c("Surv", "numeric"),
+  function(observed, predicted, weights, breaks, distr, span, ...) {
+    weights <- check_weights(weights, observed)
+    throw(check_assignment(weights))
     max_time <- max(time(observed))
     distr <- get_surv_distr(distr, observed, predicted)
     nparams <- if (distr %in% c("exponential", "rayleigh")) 1 else 2
@@ -216,17 +232,18 @@ setMethod(".calibration_default", c("Surv", "numeric"),
       tricubic <- function(x, span = 1, min_weight = 0) {
         x <- abs(x)
         x_range <- span * diff(range(x))
-        (1 - min_weight) * pmax((1 - (x / x_range)^3)^3, 0) + min_weight
+        (1 - min_weight) * pmax(0, (1 - (x / x_range)^3)^3) + min_weight
       }
       metrics_list <- map(function(value) {
-        weights <- tricubic(predicted - value, span = span, min_weight = 0.01)
+        weights <- weights * tricubic(predicted - value, span = span,
+                                      min_weight = 0.01)
         est <- if (distr == "empirical") {
           survfit_est(observed, weights)
         } else {
           survreg_est(observed, distr, weights)
         }
         with(est, c(Mean = Mean, SE = SE,
-                    Lower = max(Mean - SE, 0),
+                    Lower = max(0, Mean - SE),
                     Upper = Mean + SE))
       }, df$Predicted)
       df$Observed <- do.call(rbind, metrics_list)
@@ -235,20 +252,22 @@ setMethod(".calibration_default", c("Surv", "numeric"),
       df <- data.frame(
         Response = factor("Mean"),
         Predicted = midpoints(predicted, breaks),
-        Observed = observed
+        Observed = observed,
+        Weight = weights
       )
       by_results <- by(df, df[c("Predicted", "Response")], function(data) {
         observed <- data$Observed
+        weights <- data$Weight
         est <- if (distr == "empirical") {
-          survfit_est(observed)
+          survfit_est(observed, weights)
         } else if (length(event_time(observed)) >= nparams) {
-          survreg_est(observed, distr)
+          survreg_est(observed, distr, weights)
         } else {
           list(Mean = NA_real_, SE = NA_real_)
         }
         result <- data[1, c("Response", "Predicted")]
         result$Observed <- with(est, cbind(Mean = Mean, SE = SE,
-                                           Lower = max(Mean - SE, 0),
+                                           Lower = max(0, Mean - SE),
                                            Upper = Mean + SE))
         result
       }, simplify = FALSE)
@@ -260,8 +279,8 @@ setMethod(".calibration_default", c("Surv", "numeric"),
 
 midpoints <- function(x, breaks) {
   breaks <- if (length(breaks) == 1) {
-    break_range <- range(x, na.rm = TRUE)
-    num_breaks <- max(as.integer(breaks), 1) + 1
+    break_range <- range(x, na.rm = TRUE, finite = TRUE)
+    num_breaks <- max(1, as.integer(breaks)) + 1
     seq(break_range[1], break_range[2], length = num_breaks)
   } else {
     sort(breaks)
