@@ -8,6 +8,9 @@
 #' @param x \link[=response]{observed responses}; or \link{confusion} or
 #'   \link{resample} result containing observed and predicted responses.
 #' @param y \link[=predict]{predicted responses} if not contained in \code{x}.
+#' @param weights numeric vector of non-negative
+#'   \link[=case_weights]{case weights} for the observed \code{x} responses
+#'   [default: equal weights].
 #' @param metrics \link[=metrics]{metric} function, function name, or vector of
 #'   these with which to calculate performance.
 #' @param cutoff numeric (0, 1) threshold above which binary factor
@@ -50,58 +53,62 @@ performance <- function(x, ...) {
 #' @rdname performance
 #'
 performance.BinomialVariate <- function(
-  x, y, metrics = MachineShop::settings("metrics.numeric"), na.rm = TRUE, ...
+  x, y, weights = NULL, metrics = MachineShop::settings("metrics.numeric"),
+  na.rm = TRUE, ...
 ) {
-  .performance(x, y, metrics, na.rm, dots = list(...))
+  .performance(x, y, weights, metrics, na.rm, dots = list(...))
 }
 
 
 #' @rdname performance
 #'
 performance.factor <- function(
-  x, y, metrics = MachineShop::settings("metrics.factor"),
+  x, y, weights = NULL, metrics = MachineShop::settings("metrics.factor"),
   cutoff = MachineShop::settings("cutoff"), na.rm = TRUE, ...
 ) {
-  .performance(x, y, metrics, na.rm, cutoff = cutoff, dots = list(...))
+  .performance(x, y, weights, metrics, na.rm, cutoff = cutoff, dots = list(...))
 }
 
 
 #' @rdname performance
 #'
 performance.matrix <- function(
-  x, y, metrics = MachineShop::settings("metrics.matrix"), na.rm = TRUE, ...
+  x, y, weights = NULL, metrics = MachineShop::settings("metrics.matrix"),
+  na.rm = TRUE, ...
 ) {
-  .performance(x, y, metrics, na.rm, dots = list(...))
+  .performance(x, y, weights, metrics, na.rm, dots = list(...))
 }
 
 
 #' @rdname performance
 #'
 performance.numeric <- function(
-  x, y, metrics = MachineShop::settings("metrics.numeric"), na.rm = TRUE, ...
+  x, y, weights = NULL, metrics = MachineShop::settings("metrics.numeric"),
+  na.rm = TRUE, ...
 ) {
-  .performance(x, y, metrics, na.rm, dots = list(...))
+  .performance(x, y, weights, metrics, na.rm, dots = list(...))
 }
 
 
 #' @rdname performance
 #'
 performance.Surv <- function(
-  x, y, metrics = MachineShop::settings("metrics.Surv"),
+  x, y, weights = NULL, metrics = MachineShop::settings("metrics.Surv"),
   cutoff = MachineShop::settings("cutoff"), na.rm = TRUE, ...
 ) {
-  .performance(x, y, metrics, na.rm, cutoff = cutoff, dots = list(...))
+  .performance(x, y, weights, metrics, na.rm, cutoff = cutoff, dots = list(...))
 }
 
 
-.performance <- function(x, y, metrics, na.rm, ..., dots = NULL) {
+.performance <- function(x, y, weights, metrics, na.rm, ..., dots = NULL) {
   if (na.rm) {
-    complete <- complete_subset(x = x, y = y)
+    complete <- complete_subset(x = x, y = y, weights = weights)
     x <- complete$x
     y <- complete$y
+    weights <- complete$weights
   }
   if (length(x)) {
-    args <- list(x, y, ...)
+    args <- list(x, y, weights = weights, ...)
     metric <- if (is_one_element(metrics)) metrics[[1]] else metrics
     if (is(get0(metric), "MLMetric")) args <- c(args, dots)
     do.call(list_to_function(metrics), args)
@@ -136,37 +143,35 @@ performance.Resamples <- function(x, ...) {
   perf_list <- by(x, x$Model, function(resamples) {
     Performance(performance(x@control, resamples, ...))
   }, simplify = FALSE)
-
   do.call(c, perf_list)
 }
 
 
 performance.MLControl <- function(x, resamples, ...) {
-  perf_list <- by(resamples[c("Observed", "Predicted")], resamples$Resample,
-                  function(resample) {
-                    performance(resample[[1]], resample[[2]], ...)
-                  }, simplify = FALSE)
+  perf_list <- by(resamples, resamples$Resample, function(resample) {
+    performance(resample$Observed, resample$Predicted, resample$Weight, ...)
+  }, simplify = FALSE)
   do.call(rbind, perf_list)
 }
 
 
 performance.MLBootOptimismControl <- function(x, resamples, ...) {
-  vars <- c("Observed", "Predicted", "Boot.Observed", "Boot.Predicted",
-            "Train.Predicted")
-
   test_perf_list <- list()
   boot_perf_list <- list()
-  resamples_split <- split(resamples[vars], resamples$Resample)
+  resamples_split <- split(resamples, resamples$Resample)
   for (name in names(resamples_split)) {
     resample <- resamples_split[[name]]
     test_perf_list[[name]] <- performance(resample$Observed,
-                                          resample$Predicted, ...)
+                                          resample$Predicted,
+                                          resample$Weight, ...)
     boot_perf_list[[name]] <- performance(resample$Boot.Observed,
-                                          resample$Boot.Predicted, ...)
+                                          resample$Boot.Predicted,
+                                          resample$Weight, ...)
   }
   test_perf <- do.call(rbind, test_perf_list)
   boot_perf <- do.call(rbind, boot_perf_list)
-  train_perf <- performance(resample$Observed, resample$Train.Predicted, ...)
+  train_perf <- performance(resample$Observed, resample$Train.Predicted,
+                            resample$Weight, ...)
 
   pessimism <- test_perf - boot_perf
   sweep(pessimism, 2, train_perf, "+")
@@ -174,25 +179,20 @@ performance.MLBootOptimismControl <- function(x, resamples, ...) {
 
 
 performance.MLCVOptimismControl <- function(x, resamples, ...) {
-  vars <- c("Observed", "Predicted")
-  vars2 <- c("Resample", "Observed", "Train.Predicted",
-             paste0("CV.Predicted.", seq_len(x@folds)))
+  test_perf <- NextMethod()
 
-  resamples_split <- split(resamples[vars], resamples$Resample)
-  test_perf <- map(function(resample) {
-    performance(resample$Observed, resample$Predicted, ...)
-  }, resamples_split) %>% do.call(rbind, .)
-
-  f <- function(p, obs, pred) p * performance(obs, pred, ...)
-  resamples_factor <- ceiling(resamples$Resample / x@folds)
-  resamples_split <- split(resamples[vars2], resamples_factor)
-  cv_perf_list <- map(function(resample) {
-    p <- prop.table(table(resample$Resample))
-    Reduce("+", map(f, p, resample["Observed"], resample[-(1:3)]))
+  resamples_split <- split(resamples, ceiling(resamples$Resample / x@folds))
+  vars <- paste0("CV.Predicted.", seq_len(x@folds))
+  perf_list <- map(function(resample) {
+    f <- function(p, pred) {
+      p * performance(resample$Observed, pred, resample$Weight, ...)
+    }
+    Reduce("+", map(f, prop.table(table(resample$Resample)), resample[vars]))
   }, resamples_split)
-  cv_perf <- do.call(rbind, rep(cv_perf_list, each = x@folds))
+  cv_perf <- do.call(rbind, rep(perf_list, each = x@folds))
   train_perf <- performance(resamples_split[[1]]$Observed,
-                            resamples_split[[1]]$Train.Predicted, ...)
+                            resamples_split[[1]]$Train.Predicted,
+                            resamples_split[[1]]$Weight, ...)
 
   pessimism <- test_perf - cv_perf
   sweep(pessimism, 2, train_perf, "+")
@@ -202,7 +202,7 @@ performance.MLCVOptimismControl <- function(x, resamples, ...) {
 Performance <- function(...) {
   object <- new("Performance", ...)
   names <- c("Resample", "Metric")
-  if (length(dim(object)) == 3) names <- c(names, "Model")
+  if (ndim(object) == 3) names <- c(names, "Model")
   names(dimnames(object)) <- names
   object
 }
